// DataBlock.cpp: implementation of the CDataBlock class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "DataBlock.h"
#include "serialport.h"
#include "transferrablefile.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////


CDataBlock::CDataBlock()
{

}

CDataBlock::~CDataBlock()
{

}


bool CDataBlock::Send(CSerialPort& port, CTransferrableFile& file)
{
	unsigned char checksum;
	int a;
	int retries;
	DWORD bytesTransferred;
	unsigned char reply;

	checksum=0;
	for (a=0; a<BLOCKSIZE; a++)
	{
		checksum=(unsigned char)((checksum+m_Data[a])%256);
	}
	for (retries=0; retries<RETRIES; retries++)
	{
		if (file.ShouldAbort())
		{
			//
			//	abort transfer
			//
			port.PutBytes((unsigned char*)"\xff", 1);
			return false;
		}
		bytesTransferred=port.PutBytes((unsigned char*)"\x02", 1);
		if (bytesTransferred!=1)
		{
			return false;
		}
		bytesTransferred=port.PutBytes(m_Data, BLOCKSIZE);
		if (bytesTransferred!=BLOCKSIZE)
		{
			return false;
		}
		bytesTransferred=port.PutBytes(&checksum, 1);
		if (bytesTransferred!=1)
		{
			return false;
		}
		bytesTransferred=port.GetBytes(&reply, 1);
		if (bytesTransferred!=1)
		{
			return false;
		}
		//
		//	yaroze replies 'Y' if the block was received
		//	correctly.
		//
		if (reply=='Y')
		{
			return true;
		}
	}
	port.PutBytes((unsigned char*)"\xff", 1);
	//
	//	too many retries
	//
	return false;
}

bool CDataBlock::Receive(CSerialPort& port, CTransferrableFile& file)
{
	unsigned char checksum;
	unsigned char receivedChecksum;
	int a;
	int retries;
	DWORD bytesTransferred;

	for (retries=0; retries<RETRIES; retries++)
	{
		if (file.ShouldAbort())
		{
			//
			//	abort transfer
			//
			port.PutBytes((unsigned char*)"\xff", 1);
			return false;
		}
		bytesTransferred=port.PutBytes((unsigned char*)"\x02", 1);
		if (bytesTransferred!=1)
		{
			return false;
		}
		//
		//	When receiving a block of data, you
		//	need to flick the RTS line after every byte otherwise
		//	you dont get the next one! Just as well you don't 
		//	receive blocks very often..
		//
		for (a=0; a<BLOCKSIZE; a++)
		{
			bytesTransferred=port.GetBytes(m_Data+a, 1);
			port.FlickRTS();
			if (bytesTransferred!=1)
			{
				return false;
			}
		}
		bytesTransferred=port.GetBytes(&receivedChecksum, 1);
		if (bytesTransferred!=1)
		{
			return false;
		}
		checksum=0;
		for (a=0; a<BLOCKSIZE; a++)
		{
			checksum=(unsigned char)((checksum+m_Data[a])%256);
		}
		if (checksum==receivedChecksum)
		{
			bytesTransferred=port.PutBytes((unsigned char*)"Y", 1);
			if (bytesTransferred==1)
			{
				//
				//	block transferred correctly
				//
				return true;
			}
			else
			{
				return false;
			}
		}
		else
		{
			bytesTransferred=port.PutBytes((unsigned char*)"N", 1);
			if (bytesTransferred!=1)
			{
				return false;
			}
		}
	}
	//
	//	too many retries
	//
	port.PutBytes((unsigned char*)"\xff", 1);
	return false;
}

